// Server.cpp: implementation of the CServer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MUUDPIC.h"
#include "Server.h"
#include "ServerDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CServer::CServer()
{
}

CServer::~CServer()
{

}

bool CServer::StartServer(CServerDlg* dlg)
{
	// Start up server

  m_listen.SetDialog(dlg, this);
  m_chat.SetDialog(dlg, this);
  m_dlg = dlg;

  if (!m_listen.Create(CPORT, SOCK_STREAM, FD_READ | FD_ACCEPT)) {
    AfxMessageBox("TCP/IP create error");
    return true;
  }
  if (!m_chat.Create(BPORT, SOCK_DGRAM, FD_READ)) {
    AfxMessageBox("UDP create error");
    return true;
  }

  sockaddr_in ina;
  memset(&ina, 0, sizeof(ina));
  ina.sin_family = AF_INET;
  ina.sin_port = htons(CPORT);
  ina.sin_addr.s_addr = htonl(INADDR_ANY);

  if (m_listen.Bind((SOCKADDR*)&ina, sizeof(ina))) {
    AfxMessageBox("listen bind error");
    return true;
  }

  ina.sin_port = htons(BPORT);
  if (m_chat.Bind((SOCKADDR*)&ina, sizeof(ina))) {
    AfxMessageBox("broadcast bind error");
    return true;
  }

  if (!m_listen.Listen(5)) {
    AfxMessageBox("startup listen error");
    return true;
  }
	
  return false;
}

bool CServer::SendMsg(const CClient &cli, const CString &msg)
{
  return true;
}

bool CServer::SendOut(const CClient &cli, const CString &msg)
{
  int msglen = msg.GetLength() + 1;
  const char* cmsg = (LPCTSTR)msg;
  int sent = 0;                //Amount of data sent so far
  while (sent < msglen) {
    sent += m_chat.SendTo(&cmsg[sent], msglen-sent, (SOCKADDR*)(&cli.loc), sizeof(cli.loc));
    if (sent == SOCKET_ERROR) {
      CString temp;
      temp.Format("Error: %i", m_chat.GetLastError());
      m_dlg->DisplayLine(temp);
      return true;
    }
  }
  gout += sent;                //Update connection stats
  m_dlg->UpdateStats();  
  return false;
}

bool CServer::Broadcast(const CString& msg)
{
  m_clients.SetToStart();
  int len = m_clients.Length();
  for (int c=0; c<len; c++) {
    if (SendOut(*m_clients._GetNext(), msg))
      return true;
    m_clients._Advance();
  }
  m_dlg->DisplayLine(msg);
  return false;
}

bool CServer::SendNewClient(const CClient &cli)
{
  CString temp;
  temp.Format("%c %i %s", HDR_NEW, cli.pingtime, cli.name);
  if (Broadcast(temp))
    return true;
  temp.Format("%c* %s has joined the chat!", HDR_MSG, cli.name);
  return Broadcast(temp);
}

bool CServer::LeaveClient(CClient* cli)
{
  CString msg;
  msg.Format("%c%s", HDR_EXIT, cli->name);
  if (Broadcast(msg))
    return true;

  msg.Format("%c* %s has left the chat!", HDR_MSG, cli->name);
  if (Broadcast(msg))
    return true;

  m_clients.SetToStart();
  int l = m_clients.Length();
  for (int c=0; c<l; c++) {
    if (m_clients._GetNext() == cli)
      m_clients._DeleteThis();
    else
      m_clients._Advance();
  }
  m_dlg->UpdateClients(&m_clients);
  return false;
}

void CServer::AddClient(CClient* cli)
{
  m_clients._MoveData(cli);
  SendNewClient(*cli);
  m_dlg->UpdateClients(&m_clients);
}

LinkList<CClient>* CServer::GetClients() {
  return &m_clients;
}

void CServer::BootUser(const CString& name)
{

}

void CServer::Shutdown()
{
  CString msg;
  msg.Format("%c", HDR_SERVEXIT);
  Broadcast(msg);

  m_chat.Close();
  m_listen.Close();
}

void CServer::Talk(CClient* cli, const CString& msg)
{
  CString msg2;
  msg2.Format("%c<%s> %s", HDR_MSG, cli->name, msg);
  Broadcast(msg2);
}

CClient* CServer::Identify(const sockaddr_in &loc)
{
  m_clients.SetToStart();
  int l = m_clients.Length();
  for (int c=0; c<l; c++) {
    CClient* temp = m_clients._GetNext();
    //identify by address and port
    if ((memcmp(&temp->loc.sin_addr, &loc.sin_addr, sizeof(loc.sin_addr)) == 0)
        && (temp->loc.sin_port == loc.sin_port))
      return temp;
    else
      m_clients._Advance();
  }
  return NULL;
}
